#!/bin/bash
trap 'SIGINT_Handler' SIGINT

EXTRA_FILE="iec_extra.sh"
PRODUCT_ID=""
BMC_VERSION=""
COOKIE=".`date +%Y%m%d%H$M%S%N`.cookie"
X_CSRFTOKEN=""
R_FILE_NAME=""
N_FLAG=0
C_FLAG=0
RESET_BMC_FLAG=0
DEACTIVATE_BIOS_FLAG=0
CLOSE_SESSION_FLAG=0
ACTIVATE_ASYNC_FLAG=0

VERSION="\
------------------------------------------------------
                   Version 00.00.01
------------------------------------------------------
(C)Copyright 2020.6.19, Inventec
"

if [ -f "$EXTRA_FILE" ];then
    source "$EXTRA_FILE"
fi

function DispHelpInfo()
{
    ScriptName=`basename $0`

    echo "$VERSION"

    echo "\
Usage: $ScriptName + [options]

[Options]
    -h               Show this help info
    -v               Show version info
    -H <IP>          BMC IP
    -U <UserName>    BMC user name
    -P <Password>    BMC password
    -T <Firmware>    BMC/BIOS
    -M <Model>       entei/seaking/horsea
    -F <FileName>    BMC/BIOS firmware
    -N               Not flash ME firmware when flash BIOS (async mode)
    -C               Not preserve config when flash BMC/BIOS

Examples:
    Update horsea BMC : $ScriptName -H <IP> -U <UserName> -P <Password> -T BMC -M horsea -F rom.ima
    Update horsea BIOS: $ScriptName -H <IP> -U <UserName> -P <Password> -T BIOS -M horsea -F *.BIN
"

## save old ############### 
:<<EOF
    echo "\
Usage: $ScriptName + [options]

[Options]
    -h               Show this help info
    -v               Show version info
    -H <IP>          BMC IP
    -U <UserName>    BMC user name
    -P <Password>    BMC password
    -T <Firmware>    BMC/BIOS
    -M <Model>       entei/seaking/horsea
    -F <FileName>    BMC/BIOS firmware
    -N               Not flash ME firmware when flash BIOS (async mode)
    -C               Not preserve config when flash BMC/BIOS
    -G <log>         Collect all logs
       <async>       Show BIOS async updating progress

Examples:
    Update horsea BMC : $ScriptName -H <IP> -U <UserName> -P <Password> -T BMC -M horsea -F rom.ima
    Update horsea BIOS: $ScriptName -H <IP> -U <UserName> -P <Password> -T BIOS -M horsea -F *.BIN
    Collect Log: $ScriptName -H <IP> -U <UserName> -P <Password> -G log
"
EOF
##################################################
}

function DispErr()
{
    echo -e "\033[01;31m\n$*\033[0m"
}

function SIGINT_Handler()
{
    trap 'exit 255' SIGINT
    echo -e "\n*** Detect interrupt signal ***"
    SafeExit
}

function FreeResource()
{
    if [ "$DEACTIVATE_BIOS_FLAG" == "1" ];then
        DeactivateBIOS
        if [ $? != 0 ];then
            rm -f "$COOKIE"
            exit 255
        fi
    fi

    if [ "$RESET_BMC_FLAG" == "1" ];then
        ResetBMC
        if [ $? != 0 ];then
            rm -f "$COOKIE"
            exit 255
        fi
    fi

    if [ "$CLOSE_SESSION_FLAG" == "1" ];then
        CloseSession
        if [ $? != 0 ];then
            rm -f "$COOKIE"
            exit 255
        fi
    fi

    if [ -f "$COOKIE" ];then
        rm -f "$COOKIE"
    fi
}

function SafeExit()
{
    FreeResource
    exit 1
}

function ErrExit()
{
    FreeResource
    exit 2
}

function GetFileName()
{
    local i
    R_FILE_NAME="$1"

    if [ "$R_FILE_NAME" == "" ];then
        R_FILE_NAME="`date +%Y%m%d%H$M%S%N`"
    fi

    if [ -f "$R_FILE_NAME" ];then
        for((i = 1; i < 0xFFFF; i++))
        do
            tmp="${R_FILE_NAME}${i}"
            if [ ! -f "$tmp" ];then
                R_FILE_NAME="$tmp"
                break
            fi
        done
    fi
}

function GetDeviceID()
{
    local res
    local ProductID

    res="`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 6 1`"
   
    if [ $? != 0 ];then
        DispErr "Fail to get device id"
        return 255
    fi

    ProductID=`echo "$res" | awk '{print $10}'`
    ProductID="`echo "$res" | awk '{print $11}'`${ProductID}"
    if [ "$ProductID" == "007a" ];then
        PRODUCT_ID="arbok"
    elif [ "$ProductID" == "0075" ];then
        PRODUCT_ID="kingler"
    elif [ "$ProductID" == "0081" ];then
        PRODUCT_ID="entei"
    elif [ "$ProductID" == "0082" ];then
        PRODUCT_ID="seaking"
    elif [ "$ProductID" == "0087" ];then
        PRODUCT_ID="horsea"
    fi
    if [ $PRODUCT_ID != $MODEL ];then
        DispErr "device id mismatch: read is $PRODUCT_ID, and input is $MODEL"
        return 255
    fi

    BMC_VERSION=`echo "$res" | awk '{print $4}'`
    BMC_VER_MAJOR=`echo "$res" | awk '{print $3}'`
    BMC_VER_MAJOR="$((16#$BMC_VER_MAJOR))"
}


function CreateSession()
{
    GetFileName "$COOKIE"
    COOKIE="$R_FILE_NAME"

    X_CSRFTOKEN=`./tools/curl -s -g -X POST https://$HTTP_BMC_IP/api/session \
                      -d "password=${BMC_PASSWORD}&username=${BMC_USER_NAME}" \
                      -c "$COOKIE" -k | awk '{print $19}' `

    X_CSRFTOKEN=${X_CSRFTOKEN%\"*}
    X_CSRFTOKEN=${X_CSRFTOKEN#*\"}
    if [[ $? != 0 || "$X_CSRFTOKEN" == "" ]];then
        DispErr "Fail to create session ($BMC_IP)"
        return 255
    fi

    CLOSE_SESSION_FLAG=1
    echo "Create session ($BMC_IP) ...Done"
}

function CloseSession()
{
    local res

    res=`./tools/curl -s -X DELETE https://$HTTP_BMC_IP/api/session \
              -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
              -b "$COOKIE" -k`
    if [ $? != 0 ];then
        DispErr "Close session ($BMC_IP) ...Fail"
        return 255
    fi

    rm -f "$COOKIE"
    CLOSE_SESSION_FLAG=0
    echo "Close session ($BMC_IP) ...Done"
}

function BeforeUpload()
{
    local res
    if [[ $MODEL != "horsea" ]];then
        res=`./tools/curl -s -X GET https://$HTTP_BMC_IP/api/maintenance/fwupdate_check \
                  -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                  -b "$COOKIE" -k | awk '{print $3}'`
        if [ "$res" != "0" ];then
            if [ "$res" != "1" ];then
                DispErr "Fail to check fw update status"
                return 255
            fi

            DispErr "BIOS has been in updating"
            return 254
        fi
    fi

    echo "Prepare for upgrading ..."
    if [ "$IMAGE_TYPE" == "BMC" ];then
        ./tools/curl -X PUT https://$HTTP_BMC_IP/api/maintenance/flash \
             -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
             -b "$COOKIE" -k
        if [ $? != 0 ];then
            DispErr "Prepare for upgrading ...Fail"
            return 255
        fi
    elif [ "$IMAGE_TYPE" == "BIOS" ];then
        ./tools/curl -X PUT https://$HTTP_BMC_IP/api/maintenance/BIOSflash \
             -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
             -b "$COOKIE" -k
        if [ $? != 0 ];then
            DispErr "Prepare for upgrading ...Fail"
            return 255
        fi
    else
        DispErr "Unsupported image type"
        return 255
    fi

    echo "Prepare for upgrading ...Done"
}

function UploadFile()
{
    local res
    local FileName="$1"

    if [ "$FileName" == "" ];then
        return 255
    fi

    echo "Upload file ..."

    res=`./tools/curl -X POST https://$HTTP_BMC_IP/api/maintenance/firmware/firmware \
              -F "fwimage=@${FileName}" \
              -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
              -b "$COOKIE" -k | awk '{print $3}'`
    if [[ $? != 0 || "$res" != "0" ]];then
        DispErr "Upload file ...Fail"
        return 255
    fi

    echo "Upload file ...Done"
}

function AfterUpload()
{
    local res

    echo "Verify image ..."

    if [ "$IMAGE_TYPE" == "BMC" ];then
        res=`./tools/curl -s -X GET https://$HTTP_BMC_IP/api/maintenance/firmware/verification \
                  -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                  -b "$COOKIE" -k`
        if [ $? != 0 ];then
            DispErr "Verify image ...Fail"
            return 255
        fi
    elif [ "$IMAGE_TYPE" == "BIOS" ];then
        if [ "$ACTIVATE_ASYNC_FLAG" == "0" ];then
            res=`./tools/curl -s -X GET https://$HTTP_BMC_IP/api/maintenance/firmware/verification \
                      -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                      -b "$COOKIE" -k`
            if [ $? != 0 ];then
                DispErr "Verify image ...Fail"
                return 255
            fi
        else
            res=`./tools/curl -s -X GET https://$HTTP_BMC_IP/api/maintenance/AsyncAfterUpload\
                      -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                      -b "$COOKIE" -k`
            if [ $? != 0 ];then
                DispErr "Verify image ...Fail"
                return 255
            fi

            tmp=`echo "$res" | awk '{print $3}'`
            tmp=${tmp%%,*}
            if [ "$tmp" != "1" ];then
                DispErr "Verify image ...Fail (Internal critical error occurs)"
                return 255
            fi
        fi
    fi

    echo "Verify image ...Done"
}

function StartUpgrade()
{
    local res
    local i
    local retry=0
    local FailCnt=0

    echo "Firmware upgrade ..."

    if [ "$IMAGE_TYPE" == "BMC" ];then
        if [ $C_FLAG = 1 ];then
            res=`./tools/curl -s -X PUT https://$HTTP_BMC_IP/api/maintenance/firmware/upgrade \
                      -d '{"preserve_config":0, "flash_status":1}' \
                      -H "Content-Type:application/json" \
                      -H "X-CSRFTOKEN: $X_CSRFTOKEN" \
                      -b "$COOKIE" -k`
        else
            res=`./tools/curl -s -X PUT https://$HTTP_BMC_IP/api/maintenance/firmware/upgrade \
                    -d '{"preserve_config":1, "flash_status":1}' \
                    -H "Content-Type:application/json" \
                    -H "X-CSRFTOKEN: $X_CSRFTOKEN" \
                    -b "$COOKIE" -k`
        fi
        if [ $? != 0 ];then
            DispErr "Firmware upgrade ...Fail"
            return 255
        fi
    elif [ "$IMAGE_TYPE" == "BIOS" ];then
        res=`./tools/curl -s -X PUT https://$HTTP_BMC_IP/api/maintenance/firmware/upgrade \
                  -d '{"preserve_config":0, "flash_status":0}' \
                  -H "Content-Type:application/json" \
                  -H "X-CSRFTOKEN: $X_CSRFTOKEN" \
                  -b "$COOKIE" -k`
        if [ $? != 0 ];then
            DispErr "Firmware upgrade ...Fail"
            return 255
        fi
    else
        DispErr "Unknown image type: $IMAGE_TYPE"
        return 255
    fi

    tmp=`echo "$res" | grep -i error`
    if [ "$tmp" != "" ];then
        echo "$res"
        DispErr "Firmware upgrade ...Fail"
        return 255
    fi

    for((i = 0; i < 300; i++))
    do
        res=`./tools/curl -s -X GET https://$HTTP_BMC_IP/api/maintenance/firmware/flash-progress \
                  -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                  -b "$COOKIE" -k`
        if [ $? != 0 ];then
            FailCnt=$(($FailCnt + 1))
            if [ "$FailCnt" -gt 10 ];then
                DispErr "Fail to get upgrading progress"
                return 255
            fi

            sleep 2s
            continue
        fi

        tmp=`echo "$res" | awk '{print $7}'`
        tmp=${tmp#*\"}
        tmp=${tmp%%\%*}
        expr $tmp + 10 &>/dev/null
        if [ $? != 0 ];then
            if [[ "$tmp" =~ "Complete" ]];then
                echo -e "\rFirmware upgrade : 100%"
                break
            fi

            retry=$(($retry + 1))
            if [ "$retry" -gt 10 ];then
                DispErr "Unknown upgrading progress, res=\"$res\""
                return 255
            fi

            sleep 2s
            continue
        fi

        if [ "$tmp" == "100" ];then
            echo -e "\rFirmware upgrade : 100%"
            break
        fi

        retry=0
        FailCnt=0
        echo -ne "\rFirmware upgrade : ${tmp}%"
        sleep 2s
    done

    echo "Firmware upgrade ...Done"
}

function ResetBMC()
{
    local res

    RESET_BMC_FLAG=0
    echo "Reset BMC ..."

    res=`./tools/curl -s -X POST https://$HTTP_BMC_IP/api/maintenance/reset \
              -d '{"flag":1}' \
              -H "Content-Type:application/json" \
              -H "X-CSRFTOKEN: $X_CSRFTOKEN" \
              -b "$COOKIE" -k`
    if [[ $? != 0 || "$res" != '{ "flag": 1 }' ]];then
        DispErr "Reset BMC ...Fail"
        return 255
    fi

    echo "Reset BMC ...Done"
}

function DeactivateBIOS()
{
    DEACTIVATE_BIOS_FLAG=0
    
    if [[ $MODEL != "horsea" ]];then
        res=`./tools/curl -s -X PUT https://$HTTP_BMC_IP/api/maintenance/BIOSflash/deactivateBIOS \
                  -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                  -b "$COOKIE" -k`
        if [[ $? != 0 || "$res" != "" ]];then
            DispErr "Deactivate ...Fail"
            return 255
        fi
    fi

    res=`./tools/curl -s -X PUT https://$HTTP_BMC_IP/api/maintenance/BIOSreset \
              -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
              -b "$COOKIE" -k`
    if [[ $? != 0 || "$res" != "" ]];then
        DispErr "Reset BIOS ...Fail"
        return 255
    fi

    echo "Deactivate ...Done"
}

function AsyncGetInfo()
{
    local res
    local i
    local j

    res="`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0xa4`"
    if [ $? != 0 ];then
        return 255
    fi

    j=1
    for i in $res
    do
        if [ "$j" == 2 ];then
            R_IsMemAddrValid="$i"
        elif [ "$j" == 3 ];then
            R_IsCrc32Valid="$i"
        elif [ "$j" == 4 ];then
            R_IsTaskExist="$i"
        elif [ "$j" == 5 ];then
            R_TaskProgress="$i"
        elif [ "$j" == 6 ];then
            R_ProcessRatio="$i"
        elif [ "$j" == 8 ];then
            R_IsWaitBIOSReady="$i"
        fi

        j=$(($j + 1))
    done
}

function AsyncDispProgress()
{
    local i
    local PrevTaskProgress=""
    local FailCnt=0

    AsyncGetInfo
    if [ $? != 0 ];then
        DispErr "Async: fail to get info"
        return 255
    fi

    if [ "$R_IsMemAddrValid" == "00" ];then
        echo "No async task"
        return 255
    fi

    if [ "$R_IsTaskExist" != "01" ];then
        if [ "$R_IsCrc32Valid" == "00" ];then
            echo "FW is uploading in other way"
            return 255
        fi

        if [ "$R_TaskProgress" == "00" ];then
            echo "Wait event to trigger updating"
            return 255
        fi

        if [ "$R_IsWaitBIOSReady" == "01" ];then
            echo "Wait BIOS to be ready"
            echo -e "\033[01;32mUpdate successfully !!!\033[0m"
            return 0
        fi
    fi

    for((i = 0; i < 600; i++))
    do
        AsyncGetInfo
        if [ $? != 0 ];then
            FailCnt=$(($FailCnt + 1))
            if [ "$FailCnt" -gt 10 ];then
                DispErr "Async: fail to get info"
                return 255
            fi

            sleep 2s
            continue
        fi

        if [[ "$R_TaskProgress" != "$PrevTaskProgress" &&  "$PrevTaskProgress" != "" ]];then
            echo ""
        fi

        if [ "$R_TaskProgress" == "01" ];then
            echo -ne "\rWait ME to be stable ..."
        elif [ "$R_TaskProgress" == "02" ];then
            echo -ne "\rSwitch device ..."
        elif [ "$R_TaskProgress" == "03" ];then
            echo -ne "\rActivate flash ..."
        elif [ "$R_TaskProgress" == "04" ];then
            echo -ne "\rProtect flash ..."
        elif [ "$R_TaskProgress" == "05" ];then
            echo -ne "\rUpgrading : $((16#$R_ProcessRatio))%"
        elif [ "$R_TaskProgress" == "06" ];then
            echo -ne "\rVerify image ..."
        elif [ "$R_TaskProgress" == "07" ];then
            echo -ne "\rDeactivate flash ..."
        elif [ "$R_TaskProgress" == "08" ];then
            echo -ne "\rWait ME to be stable ..."
        elif [ "$R_TaskProgress" == "0f" ];then
            echo -ne "\rFlash successfully"
        elif [ "$R_TaskProgress" == "10" ];then
            DispErr "Error in CRC verify"
            return 255
        elif [ "$R_TaskProgress" == "11" ];then
            DispErr "Error in CRC verify"
            return 255
        elif [ "$R_TaskProgress" == "12" ];then
            DispErr "Error in switch device"
            return 255
        elif [ "$R_TaskProgress" == "13" ];then
            DispErr "Error in activate flash"
            return 255
        elif [ "$R_TaskProgress" == "14" ];then
            DispErr "Error in protect flash"
            return 255
        elif [ "$R_TaskProgress" == "15" ];then
            DispErr "Error in flash"
            return 255
        elif [ "$R_TaskProgress" == "16" ];then
            DispErr "Error in deactivate flash"
            return 255
        elif [ "$R_TaskProgress" == "1F" ];then
            DispErr "Unknown error (code: $R_TaskProgress)"
            return 255
        else
            DispErr "Unknown progress: $R_TaskProgress"
            return 255
        fi

        PrevTaskProgress="$R_TaskProgress"
        FailCnt=0

        if [ "$R_IsWaitBIOSReady" == "01" ];then
            echo -e "\nWait BIOS to be ready"
            echo -e "\033[01;32mUpdate successfully !!!\033[0m"
            break
        fi

        sleep 1s
    done
}

function MainUpdateBMC()
{
    echo "BMC FW Update >>>>>>"

    tmp=`ls -l $IMAGE | awk '{print $5}'`

    if [[ "$MODEL" == "horsea" ]];then
        if [ "$tmp" -ne "67108864" ];then  #64M
            echo "$IMAGE: damaged file"
            return 255
        fi
    elif [ "$tmp" -ne "33554432" ];then  #32M
        echo "$IMAGE: damaged file"
        return 255
    fi
    
    if [ "$C_FLAG" = "1" ];then
        echo "Disable preserving config....."
    else
        echo "Enable preserving config....."
    fi

    if [ "$MODEL" = "entei" -a "$C_FLAG" = "1" ];then
        res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x32 0xba 0x00 0x00`
        if [[ $? != 0 || "$res" != "" ]];then
            DispErr "Fail to disable preserve config function"
            return 255
        fi
    fi


    CreateSession
    if [ $? != 0 ];then
        return 3
    fi

    RESET_BMC_FLAG=1
    CLOSE_SESSION_FLAG=0

    BeforeUpload
    res="$?"
    if [ "$res" != "0" ];then
        if [ "$res" == "254" ];then
            RESET_BMC_FLAG=0
            CLOSE_SESSION_FLAG=1
        fi
        return 255
    fi

    UploadFile "$IMAGE"
    if [ $? != 0 ];then
        return 255
    fi

    AfterUpload
    if [ $? != 0 ];then
        return 255
    fi

    RESET_BMC_FLAG=0

    StartUpgrade
    if [ $? != 0 ];then
        return 255
    fi

    echo -e "\033[01;32mUpdate successfully !!!\033[0m"
}

function MainUpdateBIOSCommon()
{
    if [ "$N_FLAG" != "0" ];then
        if [ $MODEL = "entei" ];then
            res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x1a 0x01`
            if [[ $? != 0 || "$res" != "" ]];then
                DispErr "Fail to set as skipping ME firmware"
                return 255
            fi
            echo "SKip ME firmware ..."
        else
            echo "not support to SKip ME firmware ..."
        fi
    fi

    CreateSession
    if [ $? != 0 ];then
        return 3
    fi

    CLOSE_SESSION_FLAG=1

    BeforeUpload
    if [ $? != 0 ];then
        return 255
    fi

    UploadFile "$IMAGE"
    if [ $? != 0 ];then
        return 255
    fi

    AfterUpload
    if [ $? != 0 ];then
        return 255
    fi

    StartUpgrade
    if [ $? != 0 ];then
        return 255
    fi

    
    #It would deactivate automatically in 10s in BMC
    echo "Deactivate ..."
    sleep 12s

    DEACTIVATE_BIOS_FLAG=0
    DeactivateBIOS
    if [ $? != 0 ];then
        return 255
    fi

    CLOSE_SESSION_FLAG=0
    CloseSession
    if [ $? != 0 ];then
        return 255
    fi

    echo -e "\033[01;32mUpdate successfully !!!\033[0m"
}

function MainUpdateBIOSAsync()
{
    echo "Asynchronous mode >>>"
    ACTIVATE_ASYNC_FLAG=1

    if [ "$N_FLAG" != "0" ];then
        res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x1a 0x01`
        if [[ $? != 0 || "$res" != "" ]];then
            DispErr "Fail to set as skipping ME firmware"
            return 255
        fi
        echo "SKip ME firmware ..."
    fi

    CreateSession
    if [ $? != 0 ];then
        return 3
    fi

    CLOSE_SESSION_FLAG=1

    BeforeUpload
    if [ $? != 0 ];then
        return 255
    fi

    UploadFile "$IMAGE"
    if [ $? != 0 ];then
        return 255
    fi

    AfterUpload
    if [ $? != 0 ];then
        return 255
    fi

    sleep 2s

    res="`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0xa0`"  #get power on or power off
    if [ $? != 0 ];then
        DispErr "Fail to check async status"
        return 255
    fi

    if [[ "$res" =~ "00 49 50 54 01" ]];then  #power on
        echo "BIOS has been uploaded. Take effect after power off"
        return 0
    elif [[ "$res" =~ "00 49 50 54 00" ]];then #power off
        AsyncDispProgress
        if [ $? != 0 ];then
            return 255
        fi
    else                                      #unknow
        echo "Unknown state: $res"
        return 255
    fi

    return 0
}

function MainUpdateBIOS()
{
    echo "BIOS FW Update >>>>>>"
    
    tmp=`ls -l $IMAGE | awk '{print $5}'`
    
    if [[ $MODEL = "horsea" ]];then
        if [ "$tmp" -ne "16777216" ];then
            echo "$IMAGE: damaged file"
            return 255
        fi
    elif [ "$tmp" -ne "33554432" ];then
        echo "$IMAGE: damaged file"         
        return 255
    fi
    
    if [ $MODEL != "entei" ];then
        MainUpdateBIOSCommon
        res_tmp="$?"
        if [ $res_tmp = 3 ];then
            return 3
        elif [ $res_tmp != 0 ];then
            return 255
        fi
        return 0
    fi
   
## check entei power(00 49 50 54 01) on or power off(00 49 50 54 00)
    res="`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0xa0 2>/dev/null`"
    if [ $? != 0 ] || [[ ! "$res" =~ "00 49 50 54" ]];then
        MainUpdateBIOSCommon
        res_tmp="$?"
        if [ $res_tmp = 3 ];then
            return 3
        elif [ $res_tmp != 0 ];then
            return 255
        fi
        return 0
    fi


    AsyncGetInfo
    if [ $? != 0 ];then
        DispErr "Async: fail to get info"
        return 255
    fi

   ### R_IsMemAddrValid means bios is updating or not
    if [ "$R_IsMemAddrValid" == "00" ];then
        if [[ "$res" =~ "00 49 50 54 00" ]];then
            MainUpdateBIOSCommon
        elif [[ "$res" =~ "00 49 50 54 01" ]];then
            MainUpdateBIOSAsync
        else
            echo "Unknown state: $res"
            return 255
        fi
        res_tmp="$?"
        if [ $res_tmp = 3 ];then
            return 3
        elif [ $res_tmp != 0 ];then
            return 255
        fi

        return 0
    fi
    
    echo "Asynchronous mode >>>"
    AsyncDispProgress
    if [ $? != 0 ];then
        return 255
    fi
}

function KinglerCollectLog()
{
	tftp_ip_with_space="0 0 0 0";
	IPMI_CMD="ipmitool -H $BMC_IP -I lanplus -U $BMC_USER_NAME -P $BMC_PASSWORD"
	username_ascii=`echo $BMC_USER_NAME | tr -d '\n' | xxd -c 100 -i | tr -d ','`
	password_ascii=`echo $BMC_PASSWORD | tr -d '\n' | xxd -c 100 -i | tr -d ','`
	username_len=${#BMC_USER_NAME}
	password_len=${#BMC_PASSWORD}

	TRIGGER_CMD="$IPMI_CMD raw 0x2E 0x15 0xA9 0x19 0x00 \
				 $tftp_ip_with_space $username_len $password_len \
				 $username_ascii $password_ascii"
	STATE_CMD="$IPMI_CMD raw 0x2E 0x16 0xA9 0x19 0x00"
	current_state=`$STATE_CMD | cut -d ' ' -f 5`
	if [ $current_state != "ff" ]; then
	    DispErr "$BMC_IP can't running info collector now."
	    DispErr "Please try it later."
	    exit 255
	fi

	echo "Starting info collector."
	$TRIGGER_CMD > /dev/null #2>&1
	if [ $? -ne 0 ]; then
	    DispErr "Start info collector failed!"
	    exit 255
	fi

	sleep 3

	current_state_d=0
	while [ $current_state_d -lt 100 ];
	do
	    echo -en "Collecting $current_state_d% ...\r"
	    current_state=`$STATE_CMD | cut -d ' ' -f 5`
		((current_state_d=0x`echo $current_state`))
	    sleep 2
	done
	echo
	sleep 1
	current_state=`$STATE_CMD | cut -d ' ' -f 5`
	if [ $current_state != "64" ]; then
	    DispErr "Run info collector failed!"
	    exit 255
	elif [ $current_state == "64" ];then
		echo "Collecting 100% Done !"
		echo " "
	fi

    CreateSession
    if [ $? != 0 ];then
        return 3
    fi
    if [ "$FileName" == "" ];then
        res=`./tools/curl -s -X GET https//$BMC_IP/api/GetDownloadPkgName \
                     -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                     -b "$COOKIE" -k | awk '{print $3}'`
        res=${res#*\"}
        res=${res%\"*}
        if [ "$res" == "" ];then
            FileName="dump_`date +%Y%m%d-%H%M`.tar"
            echo "Set file name as $FileName"
        else
            FileName="$res"
        fi
        GetFileName "$FileName"
        FileName="$R_FILE_NAME"
    fi

    echo "Download log ..."
    res=`./tools/curl -X GET -o $FileName https://$HTTP_BMC_IP/api/DownloadOneKeyLog \
              -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
              -b "$COOKIE" -k`
    if [ $? != 0 ];then
        DispErr "Download log ...Fail"
        return 255
    fi
    echo "Download log ...Done"

	CloseSession
    if [ $? != 0 ];then
        return 255
    fi

	$IPMI_CMD raw 0x2e 0x17
}

function MainCollectLog()
{
    local ratio
    local FileName="$1"
    local res
    local i
    local FailCnt=0
    local UnknownStateCnt=0
    local RetryCnt0xFF=0

    if [ "$PRODUCT_ID" == "Kingler" ];then
        KinglerCollectLog
        if [ $? != 0 ];then
            return 255
        fi

        return 0
    fi

    if [ "$PRODUCT_ID" != "ARBOK" ];then
        DispErr "Not support in current BMC"
        return 255
    fi

    echo "Collect log ..."
    res="`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x01`"
    if [ $? != 0 ];then
        DispErr "Collect log ...Fail"
        return 255
    fi

    sleep 2s

    for((i = 0; i < 310; i++))
    do
        res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x02`
        if [ $? != 0 ];then
            FailCnt=$(($FailCnt + 1))
            if [ "$FailCnt" -gt "10" ];then
                DispErr "Collect log ...Fail"
                return 255
            fi

            DispErr "Collect log ...Retry $FailCnt"
            sleep 2s
            continue
        fi

        res=${res#*\ }
        if [ "$res" == "01" ];then
            ratio="0%"
        elif [ "$res" == "02" ];then
            ratio="5%"
        elif [ "$res" == "03" ];then
            ratio="10%"
        elif [ "$res" == "04" ];then
            ratio="15%"
        elif [ "$res" == "05" ];then
            ratio="20%"
        elif [ "$res" == "06" ];then
            ratio="30%"
        elif [ "$res" == "07" ];then
            ratio="40%"
        elif [ "$res" == "08" ];then
            ratio="50%"
        elif [ "$res" == "09" ];then
            ratio="60%"
        elif [ "$res" == "0a" ];then
            ratio="65%"
        elif [ "$res" == "10" ];then
            ratio="70%"
        elif [ "$res" == "11" ];then
            ratio="75%"
        elif [ "$res" == "20" ];then
            ratio="80%"
        elif [ "$res" == "21" ];then
            ratio="85%"
        elif [ "$res" == "22" ];then
            ratio="90%"
        elif [ "$res" == "30" ];then
            ratio="95%"
        elif [ "$res" == "00" ];then
            echo -e "\rCollect log : 100%"
            break;
        else
            if [ "$UnknownStateCnt" -lt 3 ];then
                UnknownStateCnt=$(($UnknownStateCnt + 1))
                sleep 2s
                continue
            fi

            if [ "$res" == "ff" ];then
                if [ "$RetryCnt0xFF" -ge 3 ];then
                    DispErr "Collect task may timeout in BMC, re-collect ...Fail"
                    ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x03 >/dev/null 2>&1
                    return 255
                fi

                RetryCnt0xFF=$(($RetryCnt0xFF + 1))
                echo -e "\nCollect task may timeout in BMC ...Re-collect$RetryCnt0xFF (time=$((2*$i)))"
                i=0
                FailCnt=0
                UnknownStateCnt=0
                ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x03 >/dev/null 2>&1
                sleep 3s
                ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x01 >/dev/null 2>&1
                sleep 2s
            else
                echo -e "\nUnknown state: 0x$res (time=$((2*$i)))"
                return 255
            fi
        fi

        echo -ne "\rCollect log : $ratio"
        sleep 2s
    done

    CreateSession
    if [ $? != 0 ];then
        return 255
    fi

    if [ "$i" -gt "310" ];then
        echo -e "\nCollect log ...Timeout"
        return 255
    fi

    if [ "$FileName" == "" ];then
        res=`./tools/curl -s -X GET https://$HTTP_BMC_IP/api/GetDownloadPkgName \
                  -H "X-CSRFTOKEN:$X_CSRFTOKEN" \
                  -b "$COOKIE" -k | awk '{print $3}'`
        res=${res#*\"}
        res=${res%\"*}
        if [ "$res" == "" ];then
            FileName="dump_`date +%Y%m%d-%H%M`.tar"
            echo "Set file name as $FileName"
        else
            FileName="$res"
        fi

        GetFileName "$FileName"
        FileName="$R_FILE_NAME"
    fi

    echo "Download log ..."
    res="`./tools/curl -X GET -o $FileName https://$HTTP_BMC_IP/api/ipt/download/alllogs -k`"
    if [ $? != 0 ];then
        DispErr "Download log ...Fail"
        return 255
    fi
    echo "Download log ...Done"

    CloseSession
    if [ $? != 0 ];then
        return 255
    fi

    echo "Clear BMC cache ..."
    for((i = 1; i <= 3; i++))
    do
        res="`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x03`"
        if [ $? != 0 ];then
            sleep 2s
            continue
        fi

        break
    done

    if [ "$i" -gt "3" ];then
        echo "Activate BMC to clear automatically"
    else
        echo "Clear BMC cache ...Done"
    fi

    echo "Success and save as \"$FileName\""
}

function MainClient2Server()
{
    local FileName="$1"
    local DestName
    local res

    DestName="${FileName#*\ }"
    FileName="${FileName%%\ *}"

    if [[ "$DestName" =~ "$FileName" || "$DestName" == "" ]];then
        DispErr "DestName is required"
        return 255
    fi

    if [ ! -f "$FileName" ];then
        DispErr "$FileName: no such file"
        return 255
    fi

    res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x20 0x49 0x45 0x43 0x00`
    if [[ $? != 0 || "$res" != "" ]];then
        echo "Prepare for uploading config ...Fail"
        return 255
    fi

    CreateSession
    if [ $? != 0 ];then
        return 255
    fi

    UploadFile "$FileName"

    CloseSession
    if [ $? != 0 ];then
        return 255
    fi

    res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x20 0x49 0x45 0x43 0x02 $DestName`
    if [[ $? != 0 || "$res" != "" ]];then
        echo "Enforce uploading ...Fail"
        return 255
    fi
}

function MainServer2Client()
{
    local FileName="$1"
    local SrcName
    local res

    SrcName="${FileName#*\ }"
    FileName="${FileName%%\ *}"

    if [[ "$SrcName" =~ "$FileName" || "$SrcName" == "" ]];then
        DispErr "SrcName is required"
        return 255
    fi

    if [ -f "$FileName" ];then
        DispErr "$FileName: it has existed"
        return 255
    fi

    res=`ipmitool -H $BMC_IP -U $BMC_USER_NAME -P $BMC_PASSWORD raw 0x3e 0x20 0x49 0x45 0x43 0x03 $SrcName`
    if [[ $? != 0 || "$res" != "" ]];then
        echo "Create link ...Fail"
        return 255
    fi

    CreateSession
    if [ $? != 0 ];then
        return 255
    fi

    echo "Download file ..."

    res="`./tools/curl -X GET -o $FileName https://$HTTP_BMC_IP/api/OEMDownloadFile -k`"
    if [ $? != 0 ];then
        DispErr "Download file ...Fail"
        return 255
    fi

    echo "Download file ...Done"

    CloseSession
    if [ $? != 0 ];then
        return 255
    fi
}

function MainSendRawCommand()
{
    local NetFn
    local cmd
    local data
    local DataLen
    local i

    if [ "$1" == "" ];then
        DispErr "No available parameter"
        return 255
    fi

    DataLen=0
    for i in $1
    do
        if [ "$NetFn" == "" ];then
            NetFn="$i"
            continue
        fi

        if [ "$cmd" == "" ];then
            cmd="$i"
            continue
        fi

        DataLen=1
        break
    done

    if [[ "$NetFn" == "" || "$cmd" == "" ]];then
        DispErr "NetFn and command are required"
        return 255
    fi

    if [ $DataLen == 0 ];then
        data=""
    else
        data="$1"
        data="${data#*\ }"
        data="${data#*\ }"
    fi

    echo "NetFn=$NetFn cmd=$cmd data=\"$data\""

    CreateSession
    if [ $? != 0 ];then
        return 255
    fi

    res=`./tools/curl -s -X PUT https://$HTTP_BMC_IP/api/SendRawCommand \
              -d "{\"data\":\"$1\"}" \
              -H "Content-Type:application/json" \
              -H "X-CSRFTOKEN: $X_CSRFTOKEN" \
              -b "$COOKIE" -k`

    echo $res

    CloseSession
    if [ $? != 0 ];then
        return 255
    fi
}

function successExit()
{
    exit 1
}

function failExit()
{
    exit 2
}

function timeoutExit()
{
    exit 3
}

######### run start ################

chmod 777 ./tools/*

if [[ $# == 0 ]];then 
    DispHelpInfo
    failExit
fi

while getopts "hvH:U:P:T:M:F:G:NCE:" opts
do
    case $opts in
        h )
            DispHelpInfo
            successExit
            ;;
        v )
            echo "$VERSION"
            successExit
            ;;
        H )
            BMC_IP="$OPTARG"
            ;;
        U )
            BMC_USER_NAME="$OPTARG"
            ;;
        P )
            BMC_PASSWORD="$OPTARG"
            ;;
        T )
            IMAGE_TYPE=$(echo $OPTARG | tr [a-z] [A-Z])
            ;;
        M )
            MODEL=$(echo $OPTARG | tr [A-Z] [a-z])
            ;;
        F )
            IMAGE="$OPTARG"
            ;;
        G )
            G_FLAG="$OPTARG"
            ;;
        N )
            N_FLAG=1;
            ;;
        C ) 
            C_FLAG=1;
            ;;
        E )
            E_FLAG="$OPTARG"
            ;;
        * )
            DispHelpInfo
            failExit
            ;;
    esac
done

##### not used for zijie ######
:<<EOF
if [ "$1" == "raw" ];then
    if [ $# -lt 2 ];then
        DispErr "No available data for \"raw\""
        ErrExit
    fi

    RawData="$@"
    RawData=${RawData#*\ }
    MainSendRawCommand "$RawData"
    SafeExit
fi
EOF
################################

echo "$VERSION"


if [[ "$BMC_IP" == "" || "$BMC_USER_NAME" == "" || "$BMC_PASSWORD" == "" ]];then
    DispErr "Require the option \"-H\" or \"-U\" or \"-P\""
    failExit
fi

HTTP_BMC_IP=$BMC_IP
IPV6_flag=`echo $BMC_IP | grep -Ei "([0-9a-z]{0,4}[\:]){2}[0-9a-z]"`
if [ $IPV6_flag ];then 
    HTTP_BMC_IP="[$BMC_IP]" #IPV6
fi

####### not used for zijie####
:<<EOF
if [ "$G_FLAG" == "log" ];then
    MainCollectLog
    exit 0
elif [ "$G_FLAG" == "async" ];then
    AsyncDispProgress
    exit 0
elif [ "$G_FLAG" != "" ];then
    DispErr "$G_FLAG: unknown option"
    exit 255
fi

if [ "$IMAGE" == "" ];then
    echo "Require the option \"-F\" to specify file"
    failExit
fi

if [ "$E_FLAG" == "c2s" ];then
    MainClient2Server "$IMAGE"
    if [ $? != 0 ];then
        ErrExit
    fi
    SafeExit
elif [ "$E_FLAG" == "s2c" ];then
    MainServer2Client "$IMAGE"
    if [ $? != 0 ];then
        ErrExit
    fi
    SafeExit
elif [ "$E_FLAG" != "" ];then
    DispErr "$E_FLAG: unknown option"
    exit 255
fi
EOF
##########################################

if [ ! -f "$IMAGE" ];then
    DispErr "$IMAGE: no such file"
    failExit
fi

if [ "$MODEL" != "seaking" -a "$MODEL" != "entei" -a  "$MODEL" != "horsea" ];then
    DispErr "Require the option \"-M\" to specify model\
            \nonly support seaking,horsea,entei"
    failExit
fi

GetDeviceID
if [ $? != 0 ];then
    failExit
fi
tmp=$(echo ${IMAGE##*.} | tr [A-Z] [a-z])
if [ "$IMAGE_TYPE" = "BMC" ];then
    if [[ "$tmp" = "ima" ]];then
        MainUpdateBMC
        res_tmp="$?"
        if [ $res_tmp = 3 ];then
            timeoutExit
        elif [ $res_tmp != 0 ];then
            ErrExit
        fi
    else
        DispErr "$IMAGE: not a BMC firmware\
                 \nBMC firmware name must be like xxx.ima"
        failExit
    fi
elif [ "$IMAGE_TYPE" = "BIOS" ];then
    if [[ "$tmp" = "bin" ]]|| [[ "$tmp" = "rom" ]];then
        MainUpdateBIOS
        res_tmp="$?"
        if [ $res_tmp = 3 ];then
            timeoutExit
        elif [ $res_tmp != 0 ];then
            ErrExit
        fi
    else
        DispErr "$IMAGE: not a BIOS firmware\
                \nBMC firmware name must be like xxx.BIN or xxx.rom"
        failExit
    fi
else  
    echo "Require the option \"-t\" to specify firmware"
    echo "only support BIOS and BMC firmware update"
    failExit
fi

SafeExit

